@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #ifndef _CONTEXT_H_
 #define _CONTEXT_H_
@@ -17,7 +17,8 @@ class GamePad;
 
 struct contextMeshOptions {
   // mesh algorithms
-  int optimize, optimizeNetgen, smoothCrossField, refineSteps;
+  int optimize, optimizeNetgen, refineSteps;
+  int smoothCrossField, crossFieldClosestPoint;
   double lcFactor, randFactor, randFactor3d, lcIntegrationPrecision;
   double optimizeThreshold, normals, tangents, explode, angleSmoothNormals;
   double allowSwapEdgeAngle;
@@ -157,6 +158,8 @@ public:
   int fileChooserPosition[2], extraPosition[2], extraSize[2];
   // use the system menu bar on Mac OS X?
   int systemMenuBar;
+  // show standard Gmsh menu in onelab window
+  int showModuleMenu;
   // use high-resolution opengl graphics (retina Macs)
   int highResolutionGraphics;
   // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1:
