@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <string.h>
 #include <stdlib.h>
@@ -1071,14 +1071,14 @@ GMSH_API void gmshModelMeshRenumberElements(int * ierr)
   }
 }
 
-GMSH_API void gmshModelMeshSetPeriodic(const int dim, int * tags, size_t tags_n, int * tagsSource, size_t tagsSource_n, double * affineTransformation, size_t affineTransformation_n, int * ierr)
+GMSH_API void gmshModelMeshSetPeriodic(const int dim, int * tags, size_t tags_n, int * tagsSource, size_t tagsSource_n, double * affineTransform, size_t affineTransform_n, int * ierr)
 {
   if(ierr) *ierr = 0;
   try {
     std::vector<int> api_tags_(tags, tags + tags_n);
     std::vector<int> api_tagsSource_(tagsSource, tagsSource + tagsSource_n);
-    std::vector<double> api_affineTransformation_(affineTransformation, affineTransformation + affineTransformation_n);
-    gmsh::model::mesh::setPeriodic(dim, api_tags_, api_tagsSource_, api_affineTransformation_);
+    std::vector<double> api_affineTransform_(affineTransform, affineTransform + affineTransform_n);
+    gmsh::model::mesh::setPeriodic(dim, api_tags_, api_tagsSource_, api_affineTransform_);
   }
   catch(int api_ierr_){
     if(ierr) *ierr = api_ierr_;
@@ -2262,6 +2262,23 @@ GMSH_API void gmshModelOccSymmetrize(int * dimTags, size_t dimTags_n, const doub
   }
 }
 
+GMSH_API void gmshModelOccAffineTransform(int * dimTags, size_t dimTags_n, double * a, size_t a_n, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::vectorpair api_dimTags_(dimTags_n/2);
+    for(size_t i = 0; i < dimTags_n/2; ++i){
+      api_dimTags_[i].first = dimTags[i * 2 + 0];
+      api_dimTags_[i].second = dimTags[i * 2 + 1];
+    }
+    std::vector<double> api_a_(a, a + a_n);
+    gmsh::model::occ::affineTransform(api_dimTags_, api_a_);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
 GMSH_API void gmshModelOccCopy(int * dimTags, size_t dimTags_n, int ** outDimTags, size_t * outDimTags_n, int * ierr)
 {
   if(ierr) *ierr = 0;
@@ -2547,6 +2564,50 @@ GMSH_API void gmshFltkWait(const double time, int * ierr)
   }
 }
 
+GMSH_API void gmshFltkUpdate(int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::fltk::update();
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshFltkAwake(const char * action, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::fltk::awake(action);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshFltkLock(int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::fltk::lock();
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshFltkUnlock(int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::fltk::unlock();
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
 GMSH_API void gmshFltkRun(int * ierr)
 {
   if(ierr) *ierr = 0;
@@ -2558,17 +2619,49 @@ GMSH_API void gmshFltkRun(int * ierr)
   }
 }
 
-GMSH_API void gmshOnelabGet(char ** data, const char * format, int * ierr)
+GMSH_API int gmshFltkSelectEntities(int ** dimTags, size_t * dimTags_n, const int dim, int * ierr)
 {
+  int result_api_;
   if(ierr) *ierr = 0;
   try {
-    std::string api_data_;
-    gmsh::onelab::get(api_data_, format);
-    *data = strdup(api_data_.c_str());
+    gmsh::vectorpair api_dimTags_;
+    result_api_ = gmsh::fltk::selectEntities(api_dimTags_, dim);
+    vectorpair2intptr(api_dimTags_, dimTags, dimTags_n);
   }
   catch(int api_ierr_){
     if(ierr) *ierr = api_ierr_;
   }
+  return result_api_;
+}
+
+GMSH_API int gmshFltkSelectElements(int ** tags, size_t * tags_n, int * ierr)
+{
+  int result_api_;
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<int> api_tags_;
+    result_api_ = gmsh::fltk::selectElements(api_tags_);
+    vector2ptr(api_tags_, tags, tags_n);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+  return result_api_;
+}
+
+GMSH_API int gmshFltkSelectViews(int ** tags, size_t * tags_n, int * ierr)
+{
+  int result_api_;
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<int> api_tags_;
+    result_api_ = gmsh::fltk::selectViews(api_tags_);
+    vector2ptr(api_tags_, tags, tags_n);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+  return result_api_;
 }
 
 GMSH_API void gmshOnelabSet(const char * data, const char * format, int * ierr)
@@ -2582,6 +2675,80 @@ GMSH_API void gmshOnelabSet(const char * data, const char * format, int * ierr)
   }
 }
 
+GMSH_API void gmshOnelabGet(char ** data, const char * name, const char * format, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    std::string api_data_;
+    gmsh::onelab::get(api_data_, name, format);
+    *data = strdup(api_data_.c_str());
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshOnelabSetNumber(const char * name, double * value, size_t value_n, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<double> api_value_(value, value + value_n);
+    gmsh::onelab::setNumber(name, api_value_);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshOnelabSetString(const char * name, char ** value, size_t value_n, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<std::string> api_value_(value, value + value_n);
+    gmsh::onelab::setString(name, api_value_);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshOnelabGetNumber(const char * name, double ** value, size_t * value_n, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<double> api_value_;
+    gmsh::onelab::getNumber(name, api_value_);
+    vector2ptr(api_value_, value, value_n);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshOnelabGetString(const char * name, char *** value, size_t * value_n, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    std::vector<std::string> api_value_;
+    gmsh::onelab::getString(name, api_value_);
+    vectorstring2charptrptr(api_value_, value, value_n);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
+GMSH_API void gmshOnelabClear(const char * name, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::onelab::clear(name);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
 GMSH_API void gmshOnelabRun(const char * name, const char * command, int * ierr)
 {
   if(ierr) *ierr = 0;
@@ -2593,6 +2760,17 @@ GMSH_API void gmshOnelabRun(const char * name, const char * command, int * ierr)
   }
 }
 
+GMSH_API void gmshLoggerWrite(const char * message, const char * level, int * ierr)
+{
+  if(ierr) *ierr = 0;
+  try {
+    gmsh::logger::write(message, level);
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+}
+
 GMSH_API void gmshLoggerStart(char *** log, size_t * log_n, int * ierr)
 {
   if(ierr) *ierr = 0;
@@ -2617,3 +2795,29 @@ GMSH_API void gmshLoggerStop(int * ierr)
   }
 }
 
+GMSH_API double gmshLoggerTime(int * ierr)
+{
+  double result_api_;
+  if(ierr) *ierr = 0;
+  try {
+    result_api_ = gmsh::logger::time();
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+  return result_api_;
+}
+
+GMSH_API double gmshLoggerCputime(int * ierr)
+{
+  double result_api_;
+  if(ierr) *ierr = 0;
+  try {
+    result_api_ = gmsh::logger::cputime();
+  }
+  catch(int api_ierr_){
+    if(ierr) *ierr = api_ierr_;
+  }
+  return result_api_;
+}
+
